//---------------------------------------------------------------------------
// TestTreePopulation2.cpp
//---------------------------------------------------------------------------
#include "TestTreePopulation2.h"
#include "SimManager.h"
#include "TreePopulation.h"
#include <cppunit/TestCaller.h>
#include <fstream>
#include <math.h>

/////////////////////////////////////////////////////////////////////////////
//  TestTreePopulation()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::TestTreePopulation()
{
  clSimManager * p_oSimManager = new clSimManager( 3, 3, "" );

  TestInitialDensities( p_oSimManager );
  TestErrorProcessing( p_oSimManager );
  TestTreeMapReading( p_oSimManager );
  TreeMapBugFixTesting(p_oSimManager);

  delete p_oSimManager;
}


/////////////////////////////////////////////////////////////////////////////
//  TreeMapBugFixTesting()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::TreeMapBugFixTesting( clSimManager * p_oSimManager )
{
  clTreePopulation * p_oPop;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  int iTreeTotal = 0;

  //Read in the parameter file
  p_oSimManager->ReadFile( WriteXMLFile9() );
  p_oPop = ( clTreePopulation * )  p_oSimManager->GetPopulationObject( "treepopulation" );

  //Count all trees - there should be 11
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {
    iTreeTotal++;
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the trees - this checks fix of bug 1 created in
  //cell 0, 14
  CPPUNIT_ASSERT( iTreeTotal == 11 );

  //Check cell 1, 14, height div 2 - this checks fix of bug 2
  iTreeTotal = 0;
  p_oTree = p_oPop->mp_oTreeShortest[1][14][3];
  while (p_oTree) {
    iTreeTotal++;
    p_oTree = p_oTree->GetTaller();
  }
  CPPUNIT_ASSERT( iTreeTotal == 6 );

}


/////////////////////////////////////////////////////////////////////////////
//  TestTreeMapReading()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::TestTreeMapReading( clSimManager * p_oSimManager )
{
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  float fX, fDbh, fHeight, fDiam10, fExpected;
  int iTreeTotal = 0;

  //Read in the parameter file
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    //Get the X - that's the tree's number
    p_oTree->GetValue( p_oPop->GetXCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fX );

    if ( fabs( 1 - fX ) < 0.01 )
    {
      //Seedling - get the diam10 and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );

      fExpected = 0.2;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 0.304380631;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 2 - fX ) < 0.01 )
    {
      //Seedling - get the diam10 and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );

      fExpected = 0.901196874;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 1.01;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 3 - fX ) < 0.01 )
    {

      //Sapling - get the diam10, DBH, and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 6.5;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 5.2052;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 6.84563436;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 4 - fX ) < 0.01 )
    {
      //Sapling - get the diam10, DBH, and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 9.99000999;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 8;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 9.461874761;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 5 - fX ) < 0.01 )
    {
      //Sapling - get the diam10, DBH, and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 2.077134764;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 1.663369519;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 3.2;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 6 - fX ) < 0.01 )
    {
      //Adult - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 15;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 15.13074207;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 7 - fX ) < 0.01 )
    {
      //Adult - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 23.15587652;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 20.4;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 8 - fX ) < 0.01 )
    {
      //Snag - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 43;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 28.93869474;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 9 - fX ) < 0.01 )
    {
      //Snag - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 15.41359121;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 15.43;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 10 - fX ) < 0.01 )
    {
      //Seedling - get the diam10 and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );

      fExpected = 0.8;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 0.909206115;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 11 - fX ) < 0.01 )
    {
      //Seedling - get the diam10 and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );

      fExpected = 0.65;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 1.05;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );

    }
    else if ( fabs( 12 - fX ) < 0.01 )
    {
      //Sapling - get the diam10, DBH, and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 5.36963037;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 );

      fExpected = 4.3;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 5.950308026;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 13 - fX ) < 0.01 )
    {
      //Sapling - get the diam10, DBH, and height and compare
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDiam10 );
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 5.6;
      CPPUNIT_ASSERT( fabs( fExpected - fDiam10 ) < 0.01 || fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 5.6;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 14 - fX ) < 0.01 )
    {
      //Adult - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 18.7;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 17.68084137;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 15 - fX ) < 0.01 )
    {
      //Adult - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 21.2;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 18.7;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 16 - fX ) < 0.01 )
    {
      //Snag - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 45.9;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 29.81422424;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else if ( fabs( 17 - fX ) < 0.01 )
    {
      //Snag - get the DBH and height and compare
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      fExpected = 32.5;
      CPPUNIT_ASSERT( fabs( fExpected - fDbh ) < 0.01 );

      fExpected = 15.8;
      CPPUNIT_ASSERT( fabs( fExpected - fHeight ) < 0.01 );
    }
    else
    {
      CppUnit::Asserter::fail( "Found unexpected tree." );
    }


    iTreeTotal++;
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the trees
  CPPUNIT_ASSERT( iTreeTotal == 17 );

}

/////////////////////////////////////////////////////////////////////////////
//  TestInitialDensities()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::TestInitialDensities( clSimManager * p_oSimManager )
{
  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch * p_oAllTrees;
  clTree * p_oTree;
  float fDbh, fHeight;
  int **p_iAdultCounts = new int*[3],
      **p_iSeedlingCounts = new int*[3], i, j;

  //Counts:  first index size class, second species
  for (i = 0; i < 3; i++) {
    p_iAdultCounts[i] = new int[3];
    p_iSeedlingCounts[i] = new int[3];
  }

  //*****************************
  // Run 1
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;
  p_oAllTrees = p_oPop->Find( "all" );
  CPPUNIT_ASSERT(NULL == p_oAllTrees->NextTree());

  //*****************************
  // Run 2
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile3() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 2!");
      }
    }
    else {
      CppUnit::Asserter::fail("Unexpected seedling for run 2!");
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 60 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 120 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 20 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 80 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 140 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 40 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 100 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 160 );

  //*****************************
  // Run 3
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile4() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
      p_iSeedlingCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 3!");
      }
    }
    else {
      //It's a seedling - get the height
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      if (fHeight <= 0.5) {
        p_iSeedlingCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fHeight > 0.5 && fHeight <= 1) {
        p_iSeedlingCounts[1][p_oTree->GetSpecies()]++;
      }
      else {
        p_iSeedlingCounts[2][p_oTree->GetSpecies()]++;
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the adult trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 60 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 120 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 20 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 80 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 140 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 40 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 100 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 160 );

  //Verify that we found all the seedlings
  //Species 1
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][0] == 72 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][0] == 72 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][0] == 96 );
  //Species 2
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][1] == 88 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][1] == 80 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][1] == 104 );
  //Species 3
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][2] == 104 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][2] == 88 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][2] == 112 );

  //*****************************
  // Run 4
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile5() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
      p_iSeedlingCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 4!");
      }
    }
    else {
      //It's a seedling - get the height
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      if (fHeight <= 0.5) {
        p_iSeedlingCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fHeight > 0.5 && fHeight <= 1) {
        p_iSeedlingCounts[1][p_oTree->GetSpecies()]++;
      }
      else {
        p_iSeedlingCounts[2][p_oTree->GetSpecies()]++;
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the adult trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 0 );

  //Verify that we found all the seedlings
  //Species 1
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][0] == 48 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][2] == 64 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][2] == 0 );

  //*****************************
  // Run 5
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile6() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
      p_iSeedlingCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 5!");
      }
    }
    else {
      //It's a seedling - get the height
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      if (fHeight <= 0.5) {
        p_iSeedlingCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fHeight > 0.5 && fHeight <= 1) {
        p_iSeedlingCounts[1][p_oTree->GetSpecies()]++;
      }
      else {
        p_iSeedlingCounts[2][p_oTree->GetSpecies()]++;
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the adult trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 0 );

  //Verify that we found all the seedlings
  //Species 1
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][1] == 80 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][2] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][2] == 88 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][2] == 0 );

  //*****************************
  // Run 6
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile7() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
      p_iSeedlingCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 6!");
      }
    }
    else {
      //It's a seedling - get the height
      p_oTree->GetValue( p_oPop->GetHeightCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fHeight );
      if (fHeight <= 0.5) {
        p_iSeedlingCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fHeight > 0.5 && fHeight <= 1) {
        p_iSeedlingCounts[1][p_oTree->GetSpecies()]++;
      }
      else {
        p_iSeedlingCounts[2][p_oTree->GetSpecies()]++;
      }
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the adult trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 0 );

  //Verify that we found all the seedlings
  //Species 1
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][0] == 96 );
  //Species 2
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][1] == 104 );
  //Species 3
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][2] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][2] == 0 );

  //*****************************
  // Run 7
  //*****************************
  p_oSimManager->ReadFile( WriteXMLFile8() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      p_iAdultCounts[i][j] = 0;
      p_iSeedlingCounts[i][j] = 0;
    }
  }

  //Find all trees
  p_oAllTrees = p_oPop->Find( "all" );
  p_oTree = p_oAllTrees->NextTree();
  while ( p_oTree )
  {

    if (p_oTree->GetType() != clTreePopulation::seedling) {
      //Get the DBH
      p_oTree->GetValue( p_oPop->GetDbhCode( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );

      if (fDbh <= 10) {
        p_iAdultCounts[0][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 10 && fDbh <= 15) {
        p_iAdultCounts[1][p_oTree->GetSpecies()]++;
      }
      else if (fDbh > 20 && fDbh <= 35) {
        p_iAdultCounts[2][p_oTree->GetSpecies()]++;
      }
      else {
       CppUnit::Asserter::fail("Unexpected DBH for run 7!");
      }
    }
    else {
      //It's a seedling - get diam10
      p_oTree->GetValue( p_oPop->GetDiam10Code( p_oTree->GetSpecies(), p_oTree->GetType() ), & fDbh );
      CPPUNIT_ASSERT(fDbh >= 0.09 && fDbh <= 0.11);
      p_iSeedlingCounts[0][p_oTree->GetSpecies()]++;
    }
    p_oTree = p_oAllTrees->NextTree();
  }

  //Verify that we found all the adult trees
  //Species 1
  CPPUNIT_ASSERT( p_iAdultCounts[0][0] == 0 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][0] == 60 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][0] == 120 );
  //Species 2
  CPPUNIT_ASSERT( p_iAdultCounts[0][1] == 20 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][1] == 80 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][1] == 140 );
  //Species 3
  CPPUNIT_ASSERT( p_iAdultCounts[0][2] == 40 );
  CPPUNIT_ASSERT( p_iAdultCounts[1][2] == 100 );
  CPPUNIT_ASSERT( p_iAdultCounts[2][2] == 160 );

  //Verify that we found all the seedlings
  //Species 1
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][0] == 24 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][0] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][0] == 0 );
  //Species 2
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][1] == 32 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][1] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][1] == 0 );
  //Species 3
  CPPUNIT_ASSERT( p_iSeedlingCounts[0][2] == 40 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[1][2] == 0 );
  CPPUNIT_ASSERT( p_iSeedlingCounts[2][2] == 0 );

  for (i = 0; i < 3; i++) {
    delete[] p_iAdultCounts[i];
    delete[] p_iSeedlingCounts[i];
  }
  delete[] p_iAdultCounts;
  delete[] p_iSeedlingCounts;

}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile5());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile5.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile5.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile6());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile6.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile6.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile7());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile7.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile7.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile8());
    CppUnit::Asserter::fail("TestInitialDensities error processing didn't catch error for WriteXMLErrorFile8.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clTreePopulation::CreateTreesFromInitialDensities") == NULL)
    {
      CppUnit::Asserter::fail("TestInitialDensities error processing caught wrong error for WriteXMLErrorFile8.");
    }
    ;
  }
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"1\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"Diam10\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"2\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Diam10\">3</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">4</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">4</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"5\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"0\" tp=\"5\">" << "<fl c=\"0\">17</fl>" //X
       << "<fl c=\"1\">17</fl>" //Y
       << "<fl c=\"2\">32.5</fl>" //DBH
       << "<fl c=\"3\">15.8</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"5\">" << "<fl c=\"0\">16</fl>" //X
       << "<fl c=\"1\">16</fl>" //Y
       << "<fl c=\"2\">45.9</fl>" //DBH
       << "<fl c=\"3\">29.81422424</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"3\">" << "<fl c=\"0\">15</fl>" //X
       << "<fl c=\"1\">15</fl>" //Y
       << "<fl c=\"2\">21.2</fl>" //DBH
       << "<fl c=\"4\">18.7</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"3\">" << "<fl c=\"0\">14</fl>" //X
       << "<fl c=\"1\">14</fl>" //Y
       << "<fl c=\"2\">18.7</fl>" //DBH
       << "<fl c=\"4\">17.68084137</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"2\">" << "<fl c=\"0\">13</fl>" //X
       << "<fl c=\"1\">13</fl>" //Y
       << "<fl c=\"3\">5.6</fl>" //diam10
       << "<fl c=\"2\">5.6</fl>" //DBH
       << "<fl c=\"4\">5.6</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"2\">" << "<fl c=\"0\">12</fl>" //X
       << "<fl c=\"1\">12</fl>" //Y
       << "<fl c=\"3\">5.36963037</fl>" //diam10
       << "<fl c=\"2\">4.3</fl>" //DBH
       << "<fl c=\"4\">5.950308026</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"1\">" << "<fl c=\"0\">11</fl>" //X
       << "<fl c=\"1\">11</fl>" //Y
       << "<fl c=\"2\">0.65</fl>" //diam10
       << "<fl c=\"3\">1.05</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"1\">" << "<fl c=\"0\">10</fl>" //X
       << "<fl c=\"1\">10</fl>" //Y
       << "<fl c=\"2\">0.8</fl>" //diam10
       << "<fl c=\"3\">0.909206115</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"5\">" << "<fl c=\"0\">9</fl>" //X
       << "<fl c=\"1\">9</fl>" //Y
       << "<fl c=\"3\">15.43</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"5\">" << "<fl c=\"0\">8</fl>" //X
       << "<fl c=\"1\">8</fl>" //Y
       << "<fl c=\"2\">43</fl>" //DBH
       << "</tree>" << "<tree sp=\"0\" tp=\"3\">" << "<fl c=\"0\">7</fl>" //X
       << "<fl c=\"1\">7</fl>" //Y
       << "<fl c=\"4\">20.4</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"3\">" << "<fl c=\"0\">6</fl>" //X
       << "<fl c=\"1\">6</fl>" //Y
       << "<fl c=\"2\">15</fl>" //DBH
       << "</tree>" << "<tree sp=\"0\" tp=\"2\">" << "<fl c=\"0\">5</fl>" //X
       << "<fl c=\"1\">5</fl>" //Y
       << "<fl c=\"4\">3.2</fl>" //Height
       << "</tree>" << "<tree sp=\"0\" tp=\"2\">" << "<fl c=\"0\">4</fl>" //X
       << "<fl c=\"1\">4</fl>" //Y
       << "<fl c=\"2\">8</fl>" //DBH
       << "</tree>" << "<tree sp=\"0\" tp=\"2\">" << "<fl c=\"0\">3</fl>" //X
       << "<fl c=\"1\">3</fl>" //Y
       << "<fl c=\"3\">6.5</fl>" //diam10
       << "</tree>" << "<tree sp=\"0\" tp=\"1\">" << "<fl c=\"0\">2</fl>" //X
       << "<fl c=\"1\">2</fl>" //Y
       << "<fl c=\"3\">1.01</fl>" //height
       << "</tree>" << "<tree sp=\"0\" tp=\"1\">" << "<fl c=\"0\">1</fl>" //X
       << "<fl c=\"1\">1</fl>" //Y
       << "<fl c=\"2\">0.2</fl>" //diam10
       << "</tree>" << "</tr_treemap>" << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">0.9</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
        << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>" << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>" << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>" << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>" << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>" << "</behaviorList>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">30.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"Seedling\" />"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">6.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">30.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">8.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedlingHeightClass1>50.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight1Density>"
       << "<tr_sh1dVal species=\"Species_1\">12</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_2\">14</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_3\">16</tr_sh1dVal>"
       << "</tr_seedlingHeight1Density>"
       << "<tr_seedlingHeight2Density>"
       << "<tr_sh2dVal species=\"Species_1\">18</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_2\">20</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_3\">22</tr_sh2dVal>"
       << "</tr_seedlingHeight2Density>"
       << "<tr_seedlingHeight3Density>"
       << "<tr_sh3dVal species=\"Species_1\">24</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_2\">26</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_3\">28</tr_sh3dVal>"
       << "</tr_seedlingHeight3Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedlingHeightClass1>50.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight1Density>"
       << "<tr_sh1dVal species=\"Species_1\">12</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_2\">0</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_3\">16</tr_sh1dVal>"
       << "</tr_seedlingHeight1Density>"
       << "<tr_seedlingHeight2Density>"
       << "<tr_sh2dVal species=\"Species_1\">0</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_2\">0</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_3\">0</tr_sh2dVal>"
       << "</tr_seedlingHeight2Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile6()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"Seedling\" />"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">0.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">0.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">0.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedlingHeightClass1>50.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight2Density>"
       << "<tr_sh2dVal species=\"Species_1\">0</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_2\">20</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_3\">22</tr_sh2dVal>"
       << "</tr_seedlingHeight2Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile7()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile7()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_seedlingHeightClass1>50.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight3Density>"
       << "<tr_sh3dVal species=\"Species_1\">24</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_2\">26</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_3\">0</tr_sh3dVal>"
       << "</tr_seedlingHeight3Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile8()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile8()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"Seedling\" />"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">6.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">30.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">8.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile9()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLFile9()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "<tr_treemap>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"2\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "<tm_floatCode label=\"Diam10\">4</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_1\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_2\" tp=\"2\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "<tm_floatCode label=\"Diam10\">4</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tm_treeSettings sp=\"Species_2\" tp=\"3\">"
       << "<tm_floatCodes>"
       << "<tm_floatCode label=\"X\">0</tm_floatCode>"
       << "<tm_floatCode label=\"Y\">1</tm_floatCode>"
       << "<tm_floatCode label=\"DBH\">2</tm_floatCode>"
       << "<tm_floatCode label=\"Height\">3</tm_floatCode>"
       << "</tm_floatCodes>"
       << "</tm_treeSettings>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">7.479825586</fl>" //X
       << "<fl c=\"1\">115.7228207</fl>" //Y
       << "<fl c=\"2\">11.4</fl>" //DBH
       << "<fl c=\"3\">5.42</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">7.479825586</fl>" //X
       << "<fl c=\"1\">117.9567383</fl>" //Y
       << "<fl c=\"2\">11.4</fl>" //DBH
       << "<fl c=\"3\">5.42</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">12.03691733</fl>" //X
       << "<fl c=\"1\">114.1195824</fl>" //Y
       << "<fl c=\"2\">44</fl>" //DBH
       << "<fl c=\"3\">15.56</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">12.03691733</fl>" //X
       << "<fl c=\"1\">119.5599766</fl>" //Y
       << "<fl c=\"2\">44</fl>" //DBH
       << "<fl c=\"3\">15.56</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">9.83690691</fl>" //X
       << "<fl c=\"1\">113.2771304</fl>" //Y
       << "<fl c=\"2\">11.3</fl>" //DBH
       << "<fl c=\"3\">8.01</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">10.94454598</fl>" //X
       << "<fl c=\"1\">114.1526108</fl>" //Y
       << "<fl c=\"2\">7</fl>" //DBH
       << "<fl c=\"3\">9.9</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"2\">"
       << "<fl c=\"0\">13.91744238</fl>" //X
       << "<fl c=\"1\">115.090028</fl>" //Y
       << "<fl c=\"2\">8.5</fl>" //DBH
       << "<fl c=\"3\">12.86</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">10.39379609</fl>" //X
       << "<fl c=\"1\">115.6548967</fl>" //Y
       << "<fl c=\"2\">17</fl>" //DBH
       << "<fl c=\"3\">15.29</fl>" //Height
       << "</tree>"
       << "<tree sp=\"0\" tp=\"3\">"
       << "<fl c=\"0\">10.39379609</fl>" //X
       << "<fl c=\"1\">118.0246623</fl>" //Y
       << "<fl c=\"2\">17</fl>" //DBH
       << "<fl c=\"3\">15.29</fl>" //Height
       << "</tree>"
       << "<tree sp=\"1\" tp=\"2\">"
       << "<fl c=\"0\">13.91744238</fl>" //X
       << "<fl c=\"1\">118.589531</fl>" //Y
       << "<fl c=\"2\">8.5</fl>" //DBH
       << "<fl c=\"3\">12.86</fl>" //Height
       << "</tree>"
       << "<tree sp=\"1\" tp=\"2\">"
       << "<fl c=\"0\">10.94454598</fl>" //X
       << "<fl c=\"1\">119.5269482</fl>" //Y
       << "<fl c=\"2\">7</fl>" //DBH
       << "<fl c=\"3\">9.9</fl>" //Height
       << "</tree>"
       << "</tr_treemap>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">21.27298</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">26.46132</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">0.9</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">0.9</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.75</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.75</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.5929285</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.3759975</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
        << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.06143158</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.04575488</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>" << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>" << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>" << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>" << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>" << "</behaviorList>" << "</paramFile>";

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile1()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">-15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile2()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">ten</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile3()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile4()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_4\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile5()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile5()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s65.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile6()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile6()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"sds\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile7()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile7()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"Seedling\" />"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">6.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">8.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedlingHeightClass1>220.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight1Density>"
       << "<tr_sh1dVal species=\"Species_1\">12</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_2\">14</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_3\">16</tr_sh1dVal>"
       << "</tr_seedlingHeight1Density>"
       << "<tr_seedlingHeight2Density>"
       << "<tr_sh2dVal species=\"Species_1\">18</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_2\">20</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_3\">22</tr_sh2dVal>"
       << "</tr_seedlingHeight2Density>"
       << "<tr_seedlingHeight3Density>"
       << "<tr_sh3dVal species=\"Species_1\">24</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_2\">26</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_3\">28</tr_sh3dVal>"
       << "</tr_seedlingHeight3Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile8()
/////////////////////////////////////////////////////////////////////////////
char * clTestTreePopulation2::WriteXMLErrorFile8()
{
  using namespace std;
  char * cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open( cFileString, ios::out | ios::trunc );

  WriteCommonPlotStuff(oOut);

  oOut << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\"/>"
       << "<tr_species speciesName=\"Species_2\"/>"
       << "<tr_species speciesName=\"Species_3\"/>"
       << "</tr_speciesList>"
       << "<tr_sizeClasses>"
       << "<tr_sizeClass sizeKey=\"Seedling\" />"
       << "<tr_sizeClass sizeKey=\"s10.0\" />"
       << "<tr_sizeClass sizeKey=\"s15.0\" />"
       << "<tr_sizeClass sizeKey=\"s20.0\" />"
       << "<tr_sizeClass sizeKey=\"s35.0\" />"
       << "</tr_sizeClasses>"
       << "<tr_initialDensities>"
       << "<tr_idVals whatSpecies=\"Species_1\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">6.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">0.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">15.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">20.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_2\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">8.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">5.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">20.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">35.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "<tr_idVals whatSpecies=\"Species_3\">"
       << "<tr_initialDensity sizeClass=\"Seedling\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s10.0\">10.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s15.0\">25.0</tr_initialDensity>"
       << "<tr_initialDensity sizeClass=\"s35.0\">40.0</tr_initialDensity>"
       << "</tr_idVals>"
       << "</tr_initialDensities>"
       << "<tr_seedlingHeightClass1>120.0</tr_seedlingHeightClass1>"
       << "<tr_seedlingHeightClass2>100.0</tr_seedlingHeightClass2>"
       << "<tr_seedlingHeight1Density>"
       << "<tr_sh1dVal species=\"Species_1\">12</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_2\">14</tr_sh1dVal>"
       << "<tr_sh1dVal species=\"Species_3\">16</tr_sh1dVal>"
       << "</tr_seedlingHeight1Density>"
       << "<tr_seedlingHeight2Density>"
       << "<tr_sh2dVal species=\"Species_1\">18</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_2\">20</tr_sh2dVal>"
       << "<tr_sh2dVal species=\"Species_3\">22</tr_sh2dVal>"
       << "</tr_seedlingHeight2Density>"
       << "<tr_seedlingHeight3Density>"
       << "<tr_sh3dVal species=\"Species_1\">24</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_2\">26</tr_sh3dVal>"
       << "<tr_sh3dVal species=\"Species_3\">28</tr_sh3dVal>"
       << "</tr_seedlingHeight3Density>";

  WriteCommonFileStuff(oOut);

  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonPlotStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::WriteCommonPlotStuff( std::fstream &oOut )
{
  using namespace std;
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
         << "<paramFile fileCode=\"06010101\">"
         << "<plot>"
         << "<timesteps>1</timesteps>"
         << "<yearsPerTimestep>1</yearsPerTimestep>"
         << "<randomSeed>1</randomSeed>"
         << "<plot_lenX>200.0</plot_lenX>"
         << "<plot_lenY>200.0</plot_lenY>"
         << "<plot_latitude>55.37</plot_latitude>"
         << "</plot>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonFileStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestTreePopulation2::WriteCommonFileStuff( std::fstream &oOut )
{
  using namespace std;
  oOut << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">0.9</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">0.9</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">0.9</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
        << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>radiallight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Snag\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_1\" type=\"Seedling\"/>"
       << "</behavior>"
       << "</behaviorList>"
       << "</paramFile>";
}

