//---------------------------------------------------------------------------
// TestNCIMort.cpp
//---------------------------------------------------------------------------
#include "TestNCIMort.h"
#include <fstream>
#include <math.h>
#include "TreePopulation.h"
#include "SimManager.h"
#include "BehaviorBase.h"
#include <iostream>

////////////////////////////////////////////////////////////////////////////
//  TestNCIMort()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestNCIMort() {
  clSimManager *p_oSimManager = new clSimManager(4, 2, "");

  TestErrorProcessing(p_oSimManager);
  TestNormalProcessing(p_oSimManager);

  delete p_oSimManager;
}

////////////////////////////////////////////////////////////////////////////
//  TestNormalProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestNormalProcessing(clSimManager *p_oSimManager) {
  using namespace std;
  cout << "Starting run 1...\n";
  cout.flush();
  TestRun1(p_oSimManager);
  cout << "Completed run 1.  Starting run 2...\n";
  cout.flush();
  TestRun2(p_oSimManager);
  cout << "Completed run 2.  Starting run 3...\n";
  cout.flush();
  TestRun3(p_oSimManager);
  cout << "Completed run 3.  Starting run 4...\n";
  cout.flush();
  TestRun4(p_oSimManager);
  cout << "Completed run 4.  Starting run 5...\n";
  cout.flush();
  TestRun5(p_oSimManager);
}

////////////////////////////////////////////////////////////////////////////
//  TestRun1()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestRun1(clSimManager *p_oSimManager) {

  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int *p_iCount; //for counting target trees
  float fDiam, fX, fY,
        fTarget1X = 10, fTarget1Y = 10, //coords of target 1 trees
        fTarget2X = 20, fTarget2Y = 20, //coords of target 2 trees
        fTarget3X = 30, fTarget3Y = 30, //coords of target 3 trees
        fTarget4X = 50, fTarget4Y = 50, //coords of target 4 trees
        fTarget5X = 53, fTarget5Y = 52, //coords of target 5 trees
        fTarget6X = 59, fTarget6Y = 50, //coords of target 6 trees
        fTarget7X = 50, fTarget7Y = 59, //coords of target 7 trees
        fTarget8X = 55, fTarget8Y = 55; //coords of target 8 trees
  int iSpecies, iDamage, i, iNumTargets = 8;
  p_iCount = new int[iNumTargets];

  //***********************************
  // Run 1 - 1 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile1() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set the damage level for each
  fDiam = 11; iSpecies = 0; iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 8;  iSpecies = 1;  iDamage = 1000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::sapling, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 10.5;  iSpecies = 0;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 1;  iDamage = 1000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 25;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget6X, fTarget6Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 30;  iSpecies = 0;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget7X, fTarget7Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 45;  iSpecies = 1;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget8X, fTarget8Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  //These trees won't compete
  p_oPop->CreateTree(56, 56, 0, clTreePopulation::snag, 30);
  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    } else if (fabs(fX - fTarget6X) < 0.001 && fabs(fY - fTarget6Y) < 0.001) {
      p_iCount[5]++;
    } else if (fabs(fX - fTarget7X) < 0.001 && fabs(fY - fTarget7Y) < 0.001) {
      p_iCount[6]++;
    } else if (fabs(fX - fTarget8X) < 0.001 && fabs(fY - fTarget8Y) < 0.001) {
      p_iCount[7]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 955.0435325)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 457.1481071)/p_iCount[1] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 609.8269821)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 405.5836563)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 320.0150515)/p_iCount[4] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[5] - 543.6630623)/p_iCount[5] < 0.5);
  CPPUNIT_ASSERT(fabs(p_iCount[6] - 107.8876174)/p_iCount[6] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[7] - 356.1306486)/p_iCount[7] < 0.1);

  //***********************************
  // Run 2 - 3 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile2() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set the damage level for each
  fDiam = 11; iSpecies = 0; iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 8;  iSpecies = 1;  iDamage = 1000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::sapling, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 10.5;  iSpecies = 0;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 1;  iDamage = 1000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 25;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget6X, fTarget6Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 30;  iSpecies = 0;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget7X, fTarget7Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 45;  iSpecies = 1;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget8X, fTarget8Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    } else if (fabs(fX - fTarget6X) < 0.001 && fabs(fY - fTarget6Y) < 0.001) {
      p_iCount[5]++;
    } else if (fabs(fX - fTarget7X) < 0.001 && fabs(fY - fTarget7Y) < 0.001) {
      p_iCount[6]++;
    } else if (fabs(fX - fTarget8X) < 0.001 && fabs(fY - fTarget8Y) < 0.001) {
      p_iCount[7]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    if (i != 6) {  //this one could indeed be 0
      CPPUNIT_ASSERT(p_iCount[i] > 0);
    }
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 871.1029885)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 95.53681915)/p_iCount[1] < 0.25);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 226.7879149)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 66.71774177)/p_iCount[3] < 0.3);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 32.77262402)/p_iCount[4] < 0.3);
  CPPUNIT_ASSERT(fabs(p_iCount[5] - 160.6902332)/p_iCount[5] < 0.5);
  CPPUNIT_ASSERT(p_iCount[6] < 10);
  CPPUNIT_ASSERT(fabs(p_iCount[7] - 45.16770788)/p_iCount[7] < 0.3);

  delete[] p_iCount;
}

////////////////////////////////////////////////////////////////////////////
//  TestRun2()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestRun2(clSimManager *p_oSimManager) {

  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int *p_iCount; //for counting target trees
  float fDiam, fX, fY,
        fTarget1X = 10, fTarget1Y = 10, //coords of target 1 trees
        fTarget2X = 50, fTarget2Y = 50; //coords of target 2 trees;
  int iSpecies, iNumTargets = 2, i;
  p_iCount = new int[iNumTargets];

  //***********************************
  // Run 1 - 1 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile3() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our two target trees
  fDiam = 11; iSpecies = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 10.5; iSpecies = 0;
  for (i = 0; i < 1000; i++) {
    p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  //Now create all the neighbors
  p_oPop->CreateTree(53, 52, 1, clTreePopulation::adult, 15);
  p_oPop->CreateTree(59, 50, 2, clTreePopulation::adult, 25);
  p_oPop->CreateTree(50, 59, 0, clTreePopulation::adult, 30);
  p_oPop->CreateTree(55, 55, 1, clTreePopulation::adult, 45);
  p_oPop->CreateTree(60, 61, 2, clTreePopulation::adult, 15);
  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 875.3906276)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 837.7646025)/p_iCount[1] < 0.1);

  //***********************************
  // Run 2 - 3 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile4() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our two target trees
  fDiam = 11; iSpecies = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 10.5; iSpecies = 0;
  for (i = 0; i < 1000; i++) {
    p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  //Now create all the neighbors
  p_oPop->CreateTree(53, 52, 1, clTreePopulation::adult, 15);
  p_oPop->CreateTree(59, 50, 2, clTreePopulation::adult, 25);
  p_oPop->CreateTree(50, 59, 0, clTreePopulation::adult, 30);
  p_oPop->CreateTree(55, 55, 1, clTreePopulation::adult, 45);
  p_oPop->CreateTree(60, 61, 2, clTreePopulation::adult, 15);
  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 670.8194984)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 587.9846918)/p_iCount[1] < 0.1);

  delete[] p_iCount;
}

////////////////////////////////////////////////////////////////////////////
//  TestRun3()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestRun3(clSimManager *p_oSimManager) {

  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int *p_iCount; //for counting target trees
  float fDiam, fX, fY, fShade,
        fTarget1X = 53, fTarget1Y = 52, //coords of target 1 trees
        fTarget2X = 59, fTarget2Y = 50, //coords of target 2 trees
        fTarget3X = 50, fTarget3Y = 59, //coords of target 3 trees
        fTarget4X = 55, fTarget4Y = 55, //coords of target 4 trees
        fTarget5X = 50, fTarget5Y = 50; //coords of target 5 trees
  int iSpecies, i, iNumTargets = 5;
  p_iCount = new int[iNumTargets];

  //***********************************
  // Run 1 - 1 year timestep
  //***********************************

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile5() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set their light levels
  fDiam = 15; iSpecies = 1; fShade = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 25;  iSpecies = 3;  fShade = 0.2;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 30;  iSpecies = 0;  fShade = 0.4;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 45;  iSpecies = 1;  fShade = 0.8;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 10.5;  iSpecies = 0;  fShade = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  //Now create all the neighbors that are too small to compete
  p_oPop->CreateTree(52, 52, 0, clTreePopulation::sapling, 9);
  p_oPop->CreateTree(53, 53, 1, clTreePopulation::adult, 11);
  p_oPop->CreateTree(54, 54, 3, clTreePopulation::adult, 13);
  p_oPop->CreateTree(56, 56, 2, clTreePopulation::adult, 15);


  //Now do the run - but don't let saillight go, or it will mess up the
  //shading values we've assigned
  for (int i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 329.7033838)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 153.080115)/p_iCount[1] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 338.9982341)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 501.471828)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 357.131635)/p_iCount[4] < 0.1);

  //***********************************
  // Run 2 - 3 year timestep
  //***********************************

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile6() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set their light levels
  fDiam = 15; iSpecies = 1; fShade = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 25;  iSpecies = 3;  fShade = 0.2;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 30;  iSpecies = 0;  fShade = 0.4;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 45;  iSpecies = 1;  fShade = 0.8;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 10.5;  iSpecies = 0;  fShade = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  //Now create all the neighbors
  p_oPop->CreateTree(52, 52, 0, clTreePopulation::sapling, 9);
  p_oPop->CreateTree(53, 53, 1, clTreePopulation::adult, 11);
  p_oPop->CreateTree(54, 54, 3, clTreePopulation::adult, 13);
  p_oPop->CreateTree(56, 56, 2, clTreePopulation::adult, 15);


  //Now do the run - but don't let saillight go, or it will mess up the
  //shading values we've assigned
  for (int i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    if (i != 1) { //this one could indeed be 0
      CPPUNIT_ASSERT(p_iCount[i] > 0);
    }
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 35.84018256)/p_iCount[0] < 0.3);
  CPPUNIT_ASSERT(p_iCount[1] < 10);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 38.95761017)/p_iCount[2] < 0.3);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 126.1071236)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 45.54964179)/p_iCount[4] < 0.3);

  delete[] p_iCount;
}

////////////////////////////////////////////////////////////////////////////
//  TestRun4()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestRun4(clSimManager *p_oSimManager) {

  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int *p_iCount; //for counting target trees
  float fDiam, fX, fY, fShade,
        fTarget1X = 53, fTarget1Y = 52, //coords of target 1 trees
        fTarget2X = 59, fTarget2Y = 50, //coords of target 2 trees
        fTarget3X = 50, fTarget3Y = 59, //coords of target 3 trees
        fTarget4X = 55, fTarget4Y = 55, //coords of target 4 trees
        fTarget5X = 50, fTarget5Y = 50; //coords of target 5 trees
  int iSpecies, i, iNumTargets = 5;
  p_iCount = new int[iNumTargets];

  //***********************************
  // Run 1 - 1 year timestep
  //***********************************

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile7() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set their light levels
  fDiam = 15; iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 25;  iSpecies = 3;  fShade = 0.2;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 30;  iSpecies = 0;  fShade = 0.4;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 45;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 10.5;  iSpecies = 0;  fShade = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  //Now do the run - but don't let saillight go, or it will mess up the
  //shading values we've assigned
  for (int i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 416.2467224)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 346.2004823)/p_iCount[1] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 338.3539099)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 606.8340522)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 346.2406171)/p_iCount[4] < 0.1);

  //***********************************
  // Run 2 - 3 year timestep
  //***********************************

  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile8() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set their light levels
  fDiam = 15; iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 25;  iSpecies = 3;  fShade = 0.2;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 30;  iSpecies = 0;  fShade = 0.4;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }

  fDiam = 45;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 10.5;  iSpecies = 0;  fShade = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetFloatDataCode("Light",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           fShade);
  }


  //Now do the run - but don't let saillight go, or it will mess up the
  //shading values we've assigned
  for (int i = 1; i < p_oSimManager->GetNumberOfBehaviors(); i++) {
    p_oSimManager->GetBehaviorObject(i)->Action();
  }

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 72.11946237)/p_iCount[0] < 0.25);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 41.49378055)/p_iCount[1] < 0.4);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 38.73589528)/p_iCount[2] < 0.3);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 223.4651633)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 41.50821325)/p_iCount[4] < 0.3);

  delete[] p_iCount;
}

////////////////////////////////////////////////////////////////////////////
//  TestRun5()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestRun5(clSimManager *p_oSimManager) {

  clPopulationBase * p_oTemp;
  clTreePopulation * p_oPop;
  clTreeSearch *p_oAllTrees;
  clTree *p_oTree;
  int *p_iCount; //for counting target trees
  float fDiam, fX, fY,
        fTarget1X = 10, fTarget1Y = 10, //coords of target 1 trees
        fTarget2X = 20, fTarget2Y = 20, //coords of target 2 trees
        fTarget3X = 30, fTarget3Y = 30, //coords of target 3 trees
        fTarget4X = 50, fTarget4Y = 50, //coords of target 4 trees
        fTarget5X = 53, fTarget5Y = 52, //coords of target 5 trees
        fTarget6X = 59, fTarget6Y = 50, //coords of target 6 trees
        fTarget7X = 50, fTarget7Y = 59, //coords of target 7 trees
        fTarget8X = 55, fTarget8Y = 55; //coords of target 8 trees
  int iSpecies, iDamage, i, iNumTargets = 8;
  p_iCount = new int[iNumTargets];

  //***********************************
  // Run 1 - 1 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile9() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set the damage level for each
  fDiam = 11; iSpecies = 0; iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 8;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::sapling, fDiam);
  }

  fDiam = 15;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 10.5;  iSpecies = 0;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 25;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget6X, fTarget6Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 30;  iSpecies = 0;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget7X, fTarget7Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 45;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget8X, fTarget8Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    } else if (fabs(fX - fTarget6X) < 0.001 && fabs(fY - fTarget6Y) < 0.001) {
      p_iCount[5]++;
    } else if (fabs(fX - fTarget7X) < 0.001 && fabs(fY - fTarget7Y) < 0.001) {
      p_iCount[6]++;
    } else if (fabs(fX - fTarget8X) < 0.001 && fabs(fY - fTarget8Y) < 0.001) {
      p_iCount[7]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    CPPUNIT_ASSERT(p_iCount[i] > 0);
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 955.0435325)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 507.9423412)/p_iCount[1] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 609.8269821)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 405.5827587)/p_iCount[3] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 320.0150515)/p_iCount[4] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[5] - 543.6439835)/p_iCount[5] < 0.5);
  CPPUNIT_ASSERT(fabs(p_iCount[6] - 107.8860835)/p_iCount[6] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[7] - 356.111574)/p_iCount[7] < 0.1);

  //***********************************
  // Run 2 - 3 year timestep
  //***********************************
  //Load our test file and get our tree population pointers
  p_oSimManager->ReadFile( WriteXMLFile10() );
  p_oTemp = p_oSimManager->GetPopulationObject( "treepopulation" );
  p_oPop = ( clTreePopulation * ) p_oTemp;

  //Create our trees and set the damage level for each
  fDiam = 11; iSpecies = 0; iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget1X, fTarget1Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 8;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget2X, fTarget2Y, iSpecies, clTreePopulation::sapling, fDiam);
  }

  fDiam = 15;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget3X, fTarget3Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 10.5;  iSpecies = 0;  iDamage = 0;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget4X, fTarget4Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 15;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget5X, fTarget5Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  fDiam = 25;  iSpecies = 3;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget6X, fTarget6Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 30;  iSpecies = 0;  iDamage = 2000;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget7X, fTarget7Y, iSpecies, clTreePopulation::adult, fDiam);
    p_oTree->SetValue(p_oPop->GetIntDataCode("stm_dmg",
                                                  p_oTree->GetSpecies(),
                                                  p_oTree->GetType()),
                           iDamage);
  }

  fDiam = 45;  iSpecies = 1;
  for (i = 0; i < 1000; i++) {
    p_oTree = p_oPop->CreateTree(fTarget8X, fTarget8Y, iSpecies, clTreePopulation::adult, fDiam);
  }

  p_oPop->CreateTree(56, 56, 0, clTreePopulation::seedling, 0.2);

  //Now do the run
  p_oSimManager->RunSim(1);

  //Count the target trees that are left
  for (i = 0; i < iNumTargets; i++) {
    p_iCount[i] = 0;
  }

  p_oAllTrees = p_oPop->Find("all");

  p_oTree = p_oAllTrees->NextTree();
  while (p_oTree) {

    //Get the X and Y
    p_oTree->GetValue(p_oPop->GetXCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fX);
    p_oTree->GetValue(p_oPop->GetYCode(p_oTree->GetSpecies(), p_oTree->GetType()), &fY);

    //Find out if it's a target tree
    if (fabs(fX - fTarget1X) < 0.001 && fabs(fY - fTarget1Y) < 0.001) {
      p_iCount[0]++;
    } else if (fabs(fX - fTarget2X) < 0.001 && fabs(fY - fTarget2Y) < 0.001) {
      p_iCount[1]++;
    } else if (fabs(fX - fTarget3X) < 0.001 && fabs(fY - fTarget3Y) < 0.001) {
      p_iCount[2]++;
    } else if (fabs(fX - fTarget4X) < 0.001 && fabs(fY - fTarget4Y) < 0.001) {
      p_iCount[3]++;
    } else if (fabs(fX - fTarget5X) < 0.001 && fabs(fY - fTarget5Y) < 0.001) {
      p_iCount[4]++;
    } else if (fabs(fX - fTarget6X) < 0.001 && fabs(fY - fTarget6Y) < 0.001) {
      p_iCount[5]++;
    } else if (fabs(fX - fTarget7X) < 0.001 && fabs(fY - fTarget7Y) < 0.001) {
      p_iCount[6]++;
    } else if (fabs(fX - fTarget8X) < 0.001 && fabs(fY - fTarget8Y) < 0.001) {
      p_iCount[7]++;
    }

    p_oTree = p_oAllTrees->NextTree();
  }

  //Check our counts
  //Make sure there are no divide by zero errors
  for (i = 0; i < iNumTargets; i++) {
    if (i != 6) {
      CPPUNIT_ASSERT(p_iCount[i] > 0);
    }
  }
  CPPUNIT_ASSERT(fabs(p_iCount[0] - 871.1029885)/p_iCount[0] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[1] - 131.0518781)/p_iCount[1] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[2] - 226.7879149)/p_iCount[2] < 0.1);
  CPPUNIT_ASSERT(fabs(p_iCount[3] - 66.71729881)/p_iCount[3] < 0.3);
  CPPUNIT_ASSERT(fabs(p_iCount[4] - 32.77262402)/p_iCount[4] < 0.4);
  CPPUNIT_ASSERT(fabs(p_iCount[5] - 160.6733165)/p_iCount[5] < 0.1);
  CPPUNIT_ASSERT(p_iCount[6] < 10);
  CPPUNIT_ASSERT(fabs(p_iCount[7] - 45.16045061)/p_iCount[7] < 0.3);

  delete[] p_iCount;
}

////////////////////////////////////////////////////////////////////////////
//  TestErrorProcessing()
////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::TestErrorProcessing(clSimManager *p_oSimManager) {

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile1());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile1.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile1.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile2());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile2.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile2.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile3());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile3.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile3.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile4());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile4.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile4.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile5());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile5.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile5.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile6());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile6.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile6.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile7());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile7.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile7.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile8());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile8.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile8.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile9());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile9.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile9.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile10());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile10.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile10.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile11());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile11.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile11.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile12());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile12.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::ValidateData") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile12.");
    }
    ;
  }

  try
  {
    p_oSimManager->ReadFile(WriteXMLErrorFile13());
    CppUnit::Asserter::fail("TestNCIMortality error processing didn't catch error for WriteXMLErrorFile13.");
  }
  catch (modelErr err)
  {
    if (strstr(err.cFunction, "clNCIMort::GetTreeMemberCodes") == NULL)
    {
      CppUnit::Asserter::fail("TestNCIMortality error processing caught wrong error for WriteXMLErrorFile13.");
    }
    ;
  }

}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">-10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}


////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile1()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile1() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteRun1CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile2() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteRun1CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile3()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile3() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteRun2CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile4()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile4() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteRun2CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile5()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile5() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteRun3CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile6()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile6() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteRun3CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile7()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile7() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteRun4CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile8()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile8() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteRun4CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile9()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile9() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteRun5CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile10()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLFile10() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>2</timesteps>"
       << "<yearsPerTimestep>3</yearsPerTimestep>";

  WriteRun5CommonStuff(oOut);

  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile2()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile2() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">-0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile3()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile3() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">0</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile4()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile4() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">0</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile5()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile5() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">-0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile6()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile6() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">1.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile7()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile7() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">-0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile8()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile8() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">1.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile10()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile10() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">1.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile11()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile11() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">-0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile12()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile12() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">1.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLFile9()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile9() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.43</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.2</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">34.2</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.98</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1.63</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">1.63</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">-0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.42</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.54</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
        << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
        << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
        << "</mo_prnciSpecies_1NeighborLambda>"
        << "<mo_prnciSpecies_2NeighborLambda>"
        << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
        << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
        << "</mo_prnciSpecies_2NeighborLambda>"
        << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";


  oOut.close();

  return cFileString;
}

/////////////////////////////////////////////////////////////////////////////
//  WritePlotAndTrees()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WritePlotAndTrees( std::fstream &oOut )
{
  using namespace std;
  oOut << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "<tr_species speciesName=\"Species_3\" />"
       << "<tr_species speciesName=\"Species_4\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_3\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_4\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_3\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_4\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_3\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_4\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_3\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_4\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_3\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_4\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_3\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_4\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_3\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_4\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_3\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_4\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_3\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_4\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_3\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_4\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_3\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_4\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_3\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_4\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_3\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_4\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_3\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_4\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_3\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_4\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_3\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_4\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_3\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_4\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_3\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_4\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteStormParameters()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteStormParameters( std::fstream &oOut )
{
  using namespace std;
  oOut << "<storm>"
       << "<st_s1ReturnInterval>1000</st_s1ReturnInterval>"
       << "<st_s2ReturnInterval>1000</st_s2ReturnInterval>"
       << "<st_s3ReturnInterval>1000</st_s3ReturnInterval>"
       << "<st_s4ReturnInterval>1000</st_s4ReturnInterval>"
       << "<st_s5ReturnInterval>1000</st_s5ReturnInterval>"
       << "<st_s6ReturnInterval>1000</st_s6ReturnInterval>"
       << "<st_s7ReturnInterval>1000</st_s7ReturnInterval>"
       << "<st_s8ReturnInterval>1000</st_s8ReturnInterval>"
       << "<st_s9ReturnInterval>1000</st_s9ReturnInterval>"
       << "<st_s10ReturnInterval>1000</st_s10ReturnInterval>"
       << "<st_susceptibility>1</st_susceptibility>"
       << "<st_stochasticity>0</st_stochasticity>"
       << "<st_minDBH>"
       << "<st_mdVal species=\"Species_1\">5</st_mdVal>"
       << "<st_mdVal species=\"Species_2\">6</st_mdVal>"
       << "<st_mdVal species=\"Species_3\">5</st_mdVal>"
       << "<st_mdVal species=\"Species_4\">6</st_mdVal>"
       << "</st_minDBH>"
       << "<st_stmDmgInterceptMed>"
       << "<st_sdimVal species=\"Species_1\">2.1789601</st_sdimVal>"
       << "<st_sdimVal species=\"Species_2\">2.475987</st_sdimVal>"
       << "<st_sdimVal species=\"Species_3\">2.1789601</st_sdimVal>"
       << "<st_sdimVal species=\"Species_4\">2.475987</st_sdimVal>"
       << "</st_stmDmgInterceptMed>"
       << "<st_stmDmgInterceptFull>"
       << "<st_sdifVal species=\"Species_1\">3.6334169</st_sdifVal>"
       << "<st_sdifVal species=\"Species_2\">3.6331242</st_sdifVal>"
       << "<st_sdifVal species=\"Species_3\">3.6334169</st_sdifVal>"
       << "<st_sdifVal species=\"Species_4\">3.6331242</st_sdifVal>"
       << "</st_stmDmgInterceptFull>"
       << "<st_stmIntensityCoeff>"
       << "<st_sicVal species=\"Species_1\">-0.3545352</st_sicVal>"
       << "<st_sicVal species=\"Species_2\">-1.2289105</st_sicVal>"
       << "<st_sicVal species=\"Species_3\">-0.3545352</st_sicVal>"
       << "<st_sicVal species=\"Species_4\">-1.2289105</st_sicVal>"
       << "</st_stmIntensityCoeff>"
       << "<st_stmDBHCoeff>"
       << "<st_sdcVal species=\"Species_1\">0.8280319</st_sdcVal>"
       << "<st_sdcVal species=\"Species_2\">0.3282479</st_sdcVal>"
       << "<st_sdcVal species=\"Species_3\">0.8280319</st_sdcVal>"
       << "<st_sdcVal species=\"Species_4\">0.3282479</st_sdcVal>"
       << "</st_stmDBHCoeff>"
       << "<st_stmPropDieMedDmg>"
       << "<st_spdmdVal species=\"Species_1\">0</st_spdmdVal>"
       << "<st_spdmdVal species=\"Species_2\">0</st_spdmdVal>"
       << "<st_spdmdVal species=\"Species_3\">0</st_spdmdVal>"
       << "<st_spdmdVal species=\"Species_4\">0</st_spdmdVal>"
       << "</st_stmPropDieMedDmg>"
       << "<st_stmPropDieFullDmg>"
       << "<st_spdfdVal species=\"Species_1\">0</st_spdfdVal>"
       << "<st_spdfdVal species=\"Species_2\">0</st_spdfdVal>"
       << "<st_spdfdVal species=\"Species_3\">0</st_spdfdVal>"
       << "<st_spdfdVal species=\"Species_4\">0</st_spdfdVal>"
       << "</st_stmPropDieFullDmg>"
       << "<st_stmPropTipUpFullDmg>"
       << "<st_sptufdVal species=\"Species_1\">0</st_sptufdVal>"
       << "<st_sptufdVal species=\"Species_2\">0</st_sptufdVal>"
       << "<st_sptufdVal species=\"Species_3\">0</st_sptufdVal>"
       << "<st_sptufdVal species=\"Species_4\">0</st_sptufdVal>"
       << "</st_stmPropTipUpFullDmg>"
       << "<st_numYearsStormSnags>0</st_numYearsStormSnags>"
       << "<st_numYearsToHeal>4</st_numYearsToHeal>"
       << "</storm>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteRun1CommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteRun1CommonStuff( std::fstream &oOut )
{
  WritePlotAndTrees(oOut);
  using namespace std;
  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>storm</behaviorName>"
       << "<version>1</version>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>storm damage applier</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  WriteStormParameters(oOut);

  oOut << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_4\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_4\">1.33</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_4\">0.09</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_4\">0.97</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.36</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_4\">-0.4</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>"
       << "<mo_psemVal species=\"Species_4\">16.43</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>"
       << "<mo_psevVal species=\"Species_4\">1.41</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">0.31</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">0.236</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_4\">0.26</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">0.00118</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">0.002009</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_4\">0.00474</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.89</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_4\">0.78</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.34</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_4\">0.63</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.83</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_4\">0.76</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_3\">0.76</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.27</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_4\">0.7</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_3\">0.7</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.08</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.03</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteRun2CommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteRun2CommonStuff( std::fstream &oOut )
{
  using namespace std;
  WritePlotAndTrees(oOut);

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>saillight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>adultstochasticmort</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_3\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_3\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "</behavior>"
       << "</behaviorList>";

  oOut << "<sailLight>"
       << "<li_heightOfFishEyePhoto>1</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_2\">0.091</li_lecVal>"
       << "<li_lecVal species=\"Species_3\">0.059</li_lecVal>"
       << "<li_lecVal species=\"Species_4\">0.059</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_maxShadingRadius>15.0</li_maxShadingRadius>"
       << "<li_skyMaskAngle>30.0</li_skyMaskAngle>"
       << "<li_crownFracOfHeight>0</li_crownFracOfHeight>"
       << "</sailLight>";

  oOut << "<mortality>"
       << "<mo_nonSizeDepMort>"
       << "<mo_nsdmVal species=\"Species_3\">0.0</mo_nsdmVal>"
       << "</mo_nonSizeDepMort>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.63</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">1</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0.2</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">0.5</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteRun3CommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteRun3CommonStuff( std::fstream &oOut )
{
  using namespace std;
  WritePlotAndTrees(oOut);

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>saillight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  oOut << "<sailLight>"
       << "<li_heightOfFishEyePhoto>1</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_2\">0.091</li_lecVal>"
       << "<li_lecVal species=\"Species_3\">0.059</li_lecVal>"
       << "<li_lecVal species=\"Species_4\">0.059</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_maxShadingRadius>15.0</li_maxShadingRadius>"
       << "<li_skyMaskAngle>30.0</li_skyMaskAngle>"
       << "<li_crownFracOfHeight>0</li_crownFracOfHeight>"
       << "</sailLight>";

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_4\">15</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_4\">1.33</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_4\">0.09</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_4\">0.97</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">-0.13</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_4\">-0.23</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.246</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>"
       << "<mo_psemVal species=\"Species_4\">16.43</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>"
       << "<mo_psevVal species=\"Species_4\">1.41</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">0.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">0.57</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_4\">0.00029</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">0.00012</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">0.00056</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_4\">1</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">1</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">1</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_4\">1</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">1</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">1</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_4\">1</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_4\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_3\">1</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_4\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_3\">1</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.664</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.08</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.0044</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.03</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.4181</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0.2</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0.3</mo_nscVal>"
       << "<mo_nscVal species=\"Species_4\">0.5</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_4\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">12</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">16</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">14</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>1</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteRun4CommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteRun4CommonStuff( std::fstream &oOut )
{
  using namespace std;
  WritePlotAndTrees(oOut);

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>saillight</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  oOut << "<sailLight>"
       << "<li_heightOfFishEyePhoto>1</li_heightOfFishEyePhoto>"
       << "<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>"
       << "<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>"
       << "<li_julianDayGrowthStarts>120</li_julianDayGrowthStarts>"
       << "<li_julianDayGrowthEnds>270</li_julianDayGrowthEnds>"
       << "<li_minSunAngle>0.785</li_minSunAngle>"
       << "<li_lightExtinctionCoefficient>"
       << "<li_lecVal species=\"Species_1\">0.08</li_lecVal>"
       << "<li_lecVal species=\"Species_2\">0.091</li_lecVal>"
       << "<li_lecVal species=\"Species_3\">0.059</li_lecVal>"
       << "<li_lecVal species=\"Species_4\">0.059</li_lecVal>"
       << "</li_lightExtinctionCoefficient>"
       << "<li_maxShadingRadius>15.0</li_maxShadingRadius>"
       << "<li_skyMaskAngle>30.0</li_skyMaskAngle>"
       << "<li_crownFracOfHeight>0</li_crownFracOfHeight>"
       << "</sailLight>";

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_4\">15</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_4\">1.33</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_4\">0.09</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_4\">0.97</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">-0.13</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_4\">-0.23</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.246</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>"
       << "<mo_psemVal species=\"Species_4\">16.43</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>"
       << "<mo_psevVal species=\"Species_4\">1.41</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">0.98</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">0.57</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_4\">0.069</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">0.012</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">0.056</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_4\">1</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">1</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">1</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_4\">1</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">1</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">1</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_4\">1</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_4\">1</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_3\">1</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_4\">1</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_3\">1</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.664</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.08</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.0044</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.03</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.4181</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0.2</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_4\">0.5</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_4\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">10</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">12</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">16</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">14</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteRun5CommonStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteRun5CommonStuff( std::fstream &oOut )
{
  using namespace std;
  WritePlotAndTrees(oOut);

  oOut << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>storm</behaviorName>"
       << "<version>1</version>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>storm damage applier</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>2</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_4\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_4\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  WriteStormParameters(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_4\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_4\">1.33</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_4\">0.09</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_4\">0.97</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.36</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_4\">-0.4</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>"
       << "<mo_psemVal species=\"Species_4\">16.43</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>"
       << "<mo_psevVal species=\"Species_4\">1.41</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">0.31</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">0.236</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_4\">0.26</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">0.00118</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">0.002009</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_4\">0.00474</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.89</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_4\">0.78</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.34</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_4\">0.63</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.83</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_4\">0.76</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_3\">0.76</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.27</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_4\">0.7</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_3\">0.7</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.08</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.03</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_4\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_4\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";
}

/////////////////////////////////////////////////////////////////////////////
//  WriteCommonErrorStuff()
/////////////////////////////////////////////////////////////////////////////
void clTestNCIMort::WriteCommonErrorStuff( std::fstream &oOut )
{
  using namespace std;
  oOut << "<randomSeed>1</randomSeed>"
       << "<plot_lenX>200.0</plot_lenX>"
       << "<plot_lenY>200.0</plot_lenY>"
       << "<plot_latitude>55.37</plot_latitude>"
       << "</plot>"
       << "<trees>"
       << "<tr_speciesList>"
       << "<tr_species speciesName=\"Species_1\" />"
       << "<tr_species speciesName=\"Species_2\" />"
       << "</tr_speciesList>"
       << "<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>"
       << "<tr_minAdultDBH>"
       << "<tr_madVal species=\"Species_1\">10.0</tr_madVal>"
       << "<tr_madVal species=\"Species_2\">10.0</tr_madVal>"
       << "</tr_minAdultDBH>"
       << "<tr_maxSeedlingHeight>"
       << "<tr_mshVal species=\"Species_1\">1.35</tr_mshVal>"
       << "<tr_mshVal species=\"Species_2\">1.35</tr_mshVal>"
       << "</tr_maxSeedlingHeight>"
       << "</trees>"
       << "<allometry>"
       << "<tr_canopyHeight>"
       << "<tr_chVal species=\"Species_1\">39.48</tr_chVal>"
       << "<tr_chVal species=\"Species_2\">39.48</tr_chVal>"
       << "</tr_canopyHeight>"
       << "<tr_stdAsympCrownRad>"
       << "<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>"
       << "<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>"
       << "</tr_stdAsympCrownRad>"
       << "<tr_stdCrownRadExp>"
       << "<tr_screVal species=\"Species_1\">1.0</tr_screVal>"
       << "<tr_screVal species=\"Species_2\">1.0</tr_screVal>"
       << "</tr_stdCrownRadExp>"
       << "<tr_conversionDiam10ToDBH>"
       << "<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>"
       << "<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>"
       << "</tr_conversionDiam10ToDBH>"
       << "<tr_interceptDiam10ToDBH>"
       << "<tr_idtdVal species=\"Species_1\">0</tr_idtdVal>"
       << "<tr_idtdVal species=\"Species_2\">0</tr_idtdVal>"
       << "</tr_interceptDiam10ToDBH>"
       << "<tr_stdAsympCrownHt>"
       << "<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>"
       << "<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>"
       << "</tr_stdAsympCrownHt>"
       << "<tr_stdCrownHtExp>"
       << "<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>"
       << "<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>"
       << "</tr_stdCrownHtExp>"
       << "<tr_slopeOfHeight-Diam10>"
       << "<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>"
       << "<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>"
       << "</tr_slopeOfHeight-Diam10>"
       << "<tr_slopeOfAsymHeight>"
       << "<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>"
       << "<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>"
       << "</tr_slopeOfAsymHeight>"
       << "<tr_whatSeedlingHeightDiam>"
       << "<tr_wsehdVal species=\"Species_1\">0</tr_wsehdVal>"
       << "<tr_wsehdVal species=\"Species_2\">0</tr_wsehdVal>"
       << "</tr_whatSeedlingHeightDiam>"
       << "<tr_whatSaplingHeightDiam>"
       << "<tr_wsahdVal species=\"Species_1\">0</tr_wsahdVal>"
       << "<tr_wsahdVal species=\"Species_2\">0</tr_wsahdVal>"
       << "</tr_whatSaplingHeightDiam>"
       << "<tr_whatAdultHeightDiam>"
       << "<tr_wahdVal species=\"Species_1\">0</tr_wahdVal>"
       << "<tr_wahdVal species=\"Species_2\">0</tr_wahdVal>"
       << "</tr_whatAdultHeightDiam>"
       << "<tr_whatAdultCrownRadDiam>"
       << "<tr_wacrdVal species=\"Species_1\">0</tr_wacrdVal>"
       << "<tr_wacrdVal species=\"Species_2\">0</tr_wacrdVal>"
       << "</tr_whatAdultCrownRadDiam>"
       << "<tr_whatAdultCrownHeightHeight>"
       << "<tr_wachhVal species=\"Species_1\">0</tr_wachhVal>"
       << "<tr_wachhVal species=\"Species_2\">0</tr_wachhVal>"
       << "</tr_whatAdultCrownHeightHeight>"
       << "<tr_whatSaplingCrownRadDiam>"
       << "<tr_wscrdVal species=\"Species_1\">0</tr_wscrdVal>"
       << "<tr_wscrdVal species=\"Species_2\">0</tr_wscrdVal>"
       << "</tr_whatSaplingCrownRadDiam>"
       << "<tr_whatSaplingCrownHeightHeight>"
       << "<tr_wschhVal species=\"Species_1\">0</tr_wschhVal>"
       << "<tr_wschhVal species=\"Species_2\">0</tr_wschhVal>"
       << "</tr_whatSaplingCrownHeightHeight>"
       << "</allometry>"
       << "<behaviorList>"
       << "<behavior>"
       << "<behaviorName>storm</behaviorName>"
       << "<version>1</version>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>storm damage applier</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>NCI Mortality</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "<behavior>"
       << "<behaviorName>removedead</behaviorName>"
       << "<version>1</version>"
       << "<applyTo species=\"Species_1\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_1\" type=\"Sapling\"/>"
       << "<applyTo species=\"Species_2\" type=\"Adult\"/>"
       << "<applyTo species=\"Species_2\" type=\"Sapling\"/>"
       << "</behavior>"
       << "</behaviorList>";

  WriteStormParameters(oOut);
}

////////////////////////////////////////////////////////////////////////////
//  WriteXMLErrorFile13()
////////////////////////////////////////////////////////////////////////////
char* clTestNCIMort::WriteXMLErrorFile13() {
  using namespace std;
  char *cFileString = "TestFile1.xml";

  //Open file to write to
  fstream oOut;
  oOut.open(cFileString, ios::out | ios::trunc);

  //Write file
  oOut << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>"
       << "<paramFile fileCode=\"06010101\">"
       << "<plot>"
       << "<timesteps>1</timesteps>"
       << "<yearsPerTimestep>1</yearsPerTimestep>";

  WriteCommonErrorStuff(oOut);

  oOut << "<mortality>"
       << "<mo_prnciMaxCrowdingRadius>"
       << "<mo_pmcrVal species=\"Species_1\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_4\">10</mo_pmcrVal>"
       << "<mo_pmcrVal species=\"Species_2\">10</mo_pmcrVal>"
       << "</mo_prnciMaxCrowdingRadius>"
       << "<mo_prnciNeighDBHEff>"
       << "<mo_pndeVal species=\"Species_1\">2.17031683</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_2\">2.81</mo_pndeVal>"
       << "<mo_pndeVal species=\"Species_4\">1.33</mo_pndeVal>"
       << "</mo_prnciNeighDBHEff>"
       << "<mo_prnciNeighDistEff>"
       << "<mo_pndseVal species=\"Species_1\">0.699942</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_2\">0.5</mo_pndseVal>"
       << "<mo_pndseVal species=\"Species_4\">0.09</mo_pndseVal>"
       << "</mo_prnciNeighDistEff>"
       << "<mo_prnciMaxPotentialSurvival>"
       << "<mo_pmpsVal species=\"Species_1\">0.98</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_2\">0.99</mo_pmpsVal>"
       << "<mo_pmpsVal species=\"Species_4\">0.97</mo_pmpsVal>"
       << "</mo_prnciMaxPotentialSurvival>"
       << "<mo_prnciSizeSensNCI>"
       << "<mo_pssnVal species=\"Species_1\">0.43</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_2\">0.36</mo_pssnVal>"
       << "<mo_pssnVal species=\"Species_4\">-0.4</mo_pssnVal>"
       << "</mo_prnciSizeSensNCI>"
       << "<mo_prnciSizeEffectMode>"
       << "<mo_psemVal species=\"Species_1\">34.24638994</mo_psemVal>"
       << "<mo_psemVal species=\"Species_2\">122.23</mo_psemVal>"
       << "<mo_psemVal species=\"Species_4\">16.43</mo_psemVal>"
       << "</mo_prnciSizeEffectMode>"
       << "<mo_prnciSizeEffectVar>"
       << "<mo_psevVal species=\"Species_1\">5</mo_psevVal>"
       << "<mo_psevVal species=\"Species_2\">2.36</mo_psevVal>"
       << "<mo_psevVal species=\"Species_4\">1.41</mo_psevVal>"
       << "</mo_prnciSizeEffectVar>"
       << "<mo_prnciNCISlope>"
       << "<mo_pnslVal species=\"Species_1\">1.63</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_2\">1.26</mo_pnslVal>"
       << "<mo_pnslVal species=\"Species_4\">1.63</mo_pnslVal>"
       << "</mo_prnciNCISlope>"
       << "<mo_prnciNCISteepness>"
       << "<mo_pnstVal species=\"Species_1\">0.9</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_2\">0.78</mo_pnstVal>"
       << "<mo_pnstVal species=\"Species_4\">0.84</mo_pnstVal>"
       << "</mo_prnciNCISteepness>"
       << "<mo_prnciStormEffMedDmg>"
       << "<mo_psemdVal species=\"Species_1\">0.89</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_2\">0.9</mo_psemdVal>"
       << "<mo_psemdVal species=\"Species_4\">0.78</mo_psemdVal>"
       << "</mo_prnciStormEffMedDmg>"
       << "<mo_prnciStormEffFullDmg>"
       << "<mo_psefdVal species=\"Species_1\">0.42</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_2\">0.34</mo_psefdVal>"
       << "<mo_psefdVal species=\"Species_4\">0.63</mo_psefdVal>"
       << "</mo_prnciStormEffFullDmg>"
       << "<mo_prnciNeighStormEffMedDmg>"
       << "<mo_pnsemdVal species=\"Species_1\">0.83</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_2\">0.33</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_4\">0.76</mo_pnsemdVal>"
       << "<mo_pnsemdVal species=\"Species_3\">0.76</mo_pnsemdVal>"
       << "</mo_prnciNeighStormEffMedDmg>"
       << "<mo_prnciNeighStormEffFullDmg>"
       << "<mo_pnsefdVal species=\"Species_1\">0.54</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_2\">0.27</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_4\">0.7</mo_pnsefdVal>"
       << "<mo_pnsefdVal species=\"Species_3\">0.7</mo_pnsefdVal>"
       << "</mo_prnciNeighStormEffFullDmg>"
       << "<mo_prnciSpecies_1NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.66401082</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.71</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.08</mo_plVal>"
       << "</mo_prnciSpecies_1NeighborLambda>"
       << "<mo_prnciSpecies_2NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.00442797</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.12</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.03</mo_plVal>"
       << "</mo_prnciSpecies_2NeighborLambda>"
       << "<mo_prnciSpecies_3NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_3NeighborLambda>"
       << "<mo_prnciSpecies_4NeighborLambda>"
       << "<mo_plVal species=\"Species_1\">0.41812471</mo_plVal>"
       << "<mo_plVal species=\"Species_2\">0.24</mo_plVal>"
       << "<mo_plVal species=\"Species_4\">0.05</mo_plVal>"
       << "</mo_prnciSpecies_4NeighborLambda>"
       << "<mo_nciShadingCoefficient>"
       << "<mo_nscVal species=\"Species_1\">0.8</mo_nscVal>"
       << "<mo_nscVal species=\"Species_2\">0</mo_nscVal>"
       << "<mo_nscVal species=\"Species_4\">0</mo_nscVal>"
       << "</mo_nciShadingCoefficient>"
       << "<mo_nciShadingExponent>"
       << "<mo_nseVal species=\"Species_1\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_2\">1</mo_nseVal>"
       << "<mo_nseVal species=\"Species_4\">1</mo_nseVal>"
       << "</mo_nciShadingExponent>"
       << "<mo_nciMinNeighborDBH>"
       << "<mo_nmndVal species=\"Species_1\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_2\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_3\">0</mo_nmndVal>"
       << "<mo_nmndVal species=\"Species_4\">0</mo_nmndVal>"
       << "</mo_nciMinNeighborDBH>"
       << "<mo_nciDbhDivisor>100</mo_nciDbhDivisor>"
       << "<mo_nciIncludeSnagsInNCI>0</mo_nciIncludeSnagsInNCI>"
       << "</mortality>"
       << "</paramFile>";

  oOut.close();

  return cFileString;
}
